﻿using System;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;

namespace DTMF_IVR.Model
{
    sealed class CallSoundEngine : IDisposable
    {
        private static Speaker Speaker;
        private static Microphone Microphone;
        private static AudioMixerMediaHandler SpeakerMixer;
        private static AudioMixerMediaHandler MicropohoneMixer;
        private static MediaConnector FocusConnector;

        public static void InitMicropohoneAndSpeaker()
        {
            Speaker =  Speaker.GetDefaultDevice();
            Speaker.Start();

            Microphone = Microphone.GetDefaultDevice();
            Microphone.Start();

            FocusConnector = new MediaConnector();

            SpeakerMixer = new AudioMixerMediaHandler();
            MicropohoneMixer = new AudioMixerMediaHandler();

            FocusConnector.Connect(SpeakerMixer, Speaker);
            FocusConnector.Connect(Microphone, MicropohoneMixer);
        }

        private PhoneCallAudioReceiver CallReceiver;
        private PhoneCallAudioSender CallSender;

        public MediaConnector Connector { get; private set; }

        public AudioMixerMediaHandler IncomingAudioMixer { get; private set; }
        public AudioMixerMediaHandler OutgoingAudioMixer { get; private set; }

        public IPhoneCall ItsCall { get; private set; }

        public CallSoundEngine(IPhoneCall call)
        {
            CallReceiver = new PhoneCallAudioReceiver();
            CallSender = new PhoneCallAudioSender();

            Connector = new MediaConnector();

            IncomingAudioMixer = new AudioMixerMediaHandler();
            OutgoingAudioMixer = new AudioMixerMediaHandler();

            Connector.Connect(CallReceiver, IncomingAudioMixer);
            Connector.Connect(OutgoingAudioMixer, CallSender);

            CallReceiver.AttachToCall(call);
            CallSender.AttachToCall(call);

            ItsCall = call;
        }

        public void Focus()
        {
            FocusConnector.Connect(IncomingAudioMixer, SpeakerMixer);
            FocusConnector.Connect(MicropohoneMixer, OutgoingAudioMixer);
        }

        public void Defocus()
        {
            FocusConnector.Disconnect(IncomingAudioMixer, SpeakerMixer);
            FocusConnector.Disconnect(MicropohoneMixer, OutgoingAudioMixer);
        }

        public void Dispose()
        {
            Defocus();
            IncomingAudioMixer.Dispose();
            OutgoingAudioMixer.Dispose();
        }
    }
}
