﻿using System.Collections.Concurrent;
using System.Collections.Generic;
using DTMF_IVR.Model.InputInterpreter;
using Ozeki.VoIP;

namespace DTMF_IVR.Model.IVRMenuElements
{
    /// <summary>
    /// TODO
    /// </summary>
    internal abstract class BaseIVRMenuElement : IIVRMenuElement
    {
        public Dictionary<object, IIVRMenuElement> SubMenu { get; set; }
        protected IPhoneCall PhoneCall;
        protected Dictionary<IPhoneCall, CallSoundEngine> ItsSoundEngines;       

        protected BaseIVRMenuElement(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines,
            IInputInterpreter inputInterpreter=null)
        {
            SubMenu = new Dictionary<object, IIVRMenuElement>();
            InputInterpreter = inputInterpreter;
            ItsSoundEngines = soundEngines;
        }

        protected CallSoundEngine GetSoundEngine(IPhoneCall call)
        {
            CallSoundEngine @out = null;
            if(ItsSoundEngines.TryGetValue(call, out @out))
                return @out;

            return null;
        }

        #region IIVRMenuElement Members

        /// <summary>
        /// The current input interpreter.
        /// </summary>
        public IInputInterpreter InputInterpreter { get; set; }

        /// <summary>
        /// Add a IVRSubmenu.
        /// </summary>
        /// <param name="manuId"> TODO </param>
        /// <param name="menuElement"></param>
        /// <param name="inputInterpreter"></param>
        public void AddSubMenuElement(object manuId, IIVRMenuElement menuElement, IInputInterpreter inputInterpreter)
        {
            SubMenu.Add(manuId, menuElement);
            InputInterpreter = inputInterpreter;
        }
        
        /// <summary>
        /// Activation of the engine.
        /// </summary>
        /// <param name="phoneCall"></param>
        public abstract void EngineExecute(IPhoneCall phoneCall);

        public abstract object Clone();

        #endregion

        /// <summary>
        /// Activate InputInterpreter.
        /// </summary>
        protected void ActivateInputInterpreter()
        {
            if (InputInterpreter != null && PhoneCall != null)
            {
                InputInterpreter.InputInformation += InputInformation;
                InputInterpreter.Start(PhoneCall);
            }
        }

        /// <summary>
        /// DeActivate InputInterpreter.
        /// </summary>
        protected void DeActivateInputInterpreter()
        {
            if (InputInterpreter != null) InputInterpreter.InputInformation -= InputInformation;
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void InputInformation(object sender, InputinformationArgs e)
        {
            IVREngine.OnEngineEvent("InputInformation ", PhoneCall.DialInfo.UserName, e.Data.ToString());
            foreach (var keyValuePair in SubMenu)
            {
                if (keyValuePair.Key.Equals(e.Data))
                {
                    EngineStop();
                    keyValuePair.Value.EngineExecute(PhoneCall);
                }
            }
        }

        /// <summary>
        /// Base cloning.
        /// </summary>
        /// <param name="newWavePlayerElement"></param>
        protected void BaseCloning(BaseIVRMenuElement newWavePlayerElement)
        {
            foreach (KeyValuePair<object, IIVRMenuElement> ivrMenuElement in SubMenu)
                newWavePlayerElement.SubMenu.Add(ivrMenuElement.Key, (IIVRMenuElement) ivrMenuElement.Value.Clone());

            if (InputInterpreter != null)
                newWavePlayerElement.InputInterpreter = (IInputInterpreter) InputInterpreter.Clone();
        }

        public abstract void EngineStop();
    }
}