﻿using System;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.Common;

namespace DTMF_IVR.Model.InputInterpreter
{
    internal class DTMFInputInterpreter : IInputInterpreter
    {
        private IPhoneCall PhoneCall { get; set; }

        #region IInputInterpreter Members

        /// <summary>
        /// The input signal information.
        /// </summary>
        public event EventHandler<InputinformationArgs> InputInformation;

        /// <summary>
        /// Start the current InputInterpreter.
        /// </summary>
        /// <param name="phoneCall"></param>
        public void Start(IPhoneCall phoneCall)
        {
            PhoneCall = phoneCall;
            if (phoneCall != null)
                phoneCall.DtmfReceived += PhoneCallDtmfReceived;
        }

        /// <summary>
        /// Stop the curren InputINterpreter.
        /// </summary>
        public void Stop()
        {
            if (PhoneCall != null)
                PhoneCall.DtmfReceived -= PhoneCallDtmfReceived;
        }

        #endregion

        /// <summary>
        /// Activate InputInformation event.
        /// </summary>
        /// <param name="data"></param>
        private void OnInputInformation(object data)
        {
            if (InputInformation != null)
                InputInformation(this, new InputinformationArgs(data));
        }

        /// <summary>
        /// Handles the DTMF signals.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneCallDtmfReceived(object sender, VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>> e)
        {
            OnInputInformation(e.Item.Item2.Signal);
        }

        #region Cloning

        public object Clone()
        {
            return new DTMFInputInterpreter();
        }
        #endregion
    }
}